import { ExtensionValue, OpenApiSpec, OperationObject, ResponseObject, ParameterObject } from '@loopback/openapi-spec';
/**
 * Create a new instance of OpenApiSpecBuilder.
 *
 * @param basePath The base path on which the API is served.
 */
export declare function anOpenApiSpec(): OpenApiSpecBuilder;
/**
 * Create a new instance of OperationSpecBuilder.
 */
export declare function anOperationSpec(): OperationSpecBuilder;
export interface Extendable {
    [extension: string]: ExtensionValue;
}
export declare class BuilderBase<T extends Extendable> {
    protected _spec: T;
    constructor(initialSpec: T);
    /**
     * Add a custom (extension) property to the spec object.
     *
     * @param key The property name starting with "x-".
     * @param value The property value.
     */
    withExtension(key: string, value: ExtensionValue): this;
    /**
     * Build the spec object.
     */
    build(): T;
}
/**
 * A builder for creating OpenApiSpec documents.
 */
export declare class OpenApiSpecBuilder extends BuilderBase<OpenApiSpec> {
    /**
     * @param basePath The base path on which the API is served.
     */
    constructor();
    /**
     * Define a new OperationObject at the given path and verb (method).
     *
     * @param verb The HTTP verb.
     * @param path The path relative to basePath.
     * @param spec Additional specification of the operation.
     */
    withOperation(verb: string, path: string, spec: OperationObject | OperationSpecBuilder): this;
    /**
     * Define a new operation that returns a string response.
     *
     * @param verb The HTTP verb.
     * @param path The path relative to basePath.
     * @param operationName The name of the controller method implementing
     * this operation (`x-operation-name` field).
     */
    withOperationReturningString(verb: string, path: string, operationName?: string): this;
}
/**
 * A builder for creating OperationObject specifications.
 */
export declare class OperationSpecBuilder extends BuilderBase<OperationObject> {
    constructor();
    /**
     * Describe a response for a given HTTP status code.
     * @param status HTTP status code or string "default"
     * @param responseSpec Specification of the response
     */
    withResponse(status: number | 'default', responseSpec: ResponseObject): this;
    withStringResponse(status?: number | 'default'): this;
    /**
     * Describe a parameter accepted by the operation.
     * Note that parameters are positional in OpenAPI Spec, therefore
     * the first call of `withParameter` defines the first parameter,
     * the second call defines the second parameter, etc.
     * @param parameterSpec
     */
    withParameter(parameterSpec: ParameterObject): this;
    /**
     * Define the operation name (controller method name).
     *
     * @param name The name of the controller method implementing this operation.
     */
    withOperationName(name: string): this;
    /**
     * Describe tags associated with the operation
     * @param tags
     */
    withTags(tags: string | string[]): this;
}
